package ala2D;

import java.awt.Color;
import java.util.LinkedList;

public class DevideAndConquer extends ConvexHullAlgorithem 
{
    private Point[] p; // array of points
    //private static int n;     // number of points
    //private static int m;     // number of points of the polygon
	private LinkedList<Point> l = new LinkedList<Point>();
	private LinkedList<Line> l2 = new LinkedList<Line>();
	private MainPanel mainPanel = MainPanel.getInstance();
	private int time = 100;
	
	
    public void set(Point[] q) {
        p=q;
    }
    
    public Point[] get(){

    	Point[] result = new Point[l.size()];
    	for(int i=0;i<l.size();i++)
    		result[i] = l.get(i);
    	return result;
    }

    public void setTime(int time){
    	this.time = time;
    }

    /** exchanges two points p[i] and p[j]
     */
    private void exchange(int i, int j)
    {
        Point t=p[i];
        p[i]=p[j];
        p[j]=t;
    }


    /** computes coordinates of all points relative to point p0
     */
    private void makeRelativeTo(Point p0)
    {
        int i;
        Point p1=new Point(p0); // notwendig, weil p0 in p[] sein kann
        for (i=0; i<p.length; i++)
            p[i].makeRelativeTo(p1);
    }


    /** determines index of a point with minimum x-coordinate
     */
    private int indexOfLowestPointX(Point[] pt)
    {
        int i, min=0;
        for (i=1; i<pt.length; i++)
            if (pt[i].isLowerX(pt[min]))
                min=i;
        return min;
    }
    /** determines index of a point with maximum x-coordinate
     */
    private int indexOfHighestPointX(Point[] pt)
    {
        int i, max=0;
        for (i=1; i<pt.length; i++)
            if (pt[max].isLowerX(pt[i]))
                max=i;
        return max;
    }
    
    /** determines index of a point with minimum y-coordinate
     */
    private int indexOfLowestPointY(Point[] pt)
    {
        int i, min=0;
        for (i=1; i<pt.length; i++)
            if (pt[i].isLower(pt[min]))
                min=i;
        return min;
    }
    /** determines index of a point with maximum y-coordinate
     */
    private int indexOfHighestPointY(Point[] pt)
    {
        int i, max=0;
        for (i=1; i<pt.length; i++)
            if (pt[max].isLower(pt[i]))
                max=i;
        return max;
    }


    /** tests if the i-th node of the polygon is convex
     */
    private boolean isConvex(int i)
    {
        return p[i].relativeTo(p[i-1]).isLess(p[i+1].relativeTo(p[i-1]));
    }


    /** sorts points except p[0] according to their angle
     */
    private void sort()
    {
    	quicksort(1, p.length-1);
    }

    

    /** sorts points according to their koordinates
     */
    protected void quicksort(int lo, int hi)
    {
        int i=lo, j=hi;
        Point q=p[(lo+hi)/2];
        while (i<=j)
        {
            while (p[i].isLowerX(q)) i++;
            while (q.isLowerX(p[j])) j--;
            if (i<=j) exchange(i++, j--);
        }
        if (lo<j) quicksort(lo, j);
        if (i<hi) quicksort(i, hi);
    }

    
    private Line LowerTangent_eu(Point[] a, Point[] b)
    {
    	Line line = null;
    	Point pv = null;
    	Point q;
		
    	int p1 = indexOfLowestPointY(a);
    	int p2 = p1-1;

    	int q1 = indexOfLowestPointY(b);
    	int q2 = q1+1;
    	
    	double p1q1;
    	double p1p2;
    	double p1q2;
    	double max;
    	
    	if(p2 < 0) 			p2 = a.length-1;
    	if(q2 >= b.length)  q2 = 0;
    	
    	while(true){
    		pv = new Point(a[p1].x-10, a[p1].y, Color.CYAN);
    		q = new Point(a[p1]);
    		
    		if(a[p1].y >= b[q1].y){
	    		p1q1 = checkPoint(pv, a[p1], b[q1]);
	    		p1p2 = checkPoint(pv, a[p1], a[p2]);
	    		p1q2 = checkPoint(pv, a[p1], b[q2]);
	    		
	    		max = Math.max(p1q1, Math.max(p1p2, p1q2));
	    		
	    		if(max == p1q1){
	    			line = new Line(a[p1], b[q1], Color.CYAN);
	    			break;
	    		}
	    		else if(max == p1p2){
	    			p1 = p2;
	    			p2 = p1-1;
	    			if(p2 < 0) p2 = a.length-1;
	    		}
	    		else{
	    			q1 = q2;
	    			q2 = q1+1;
	    			if(q2 >= b.length) q2 = 0;
	    		}
    		}
    		else{
    			if(a[p1].equals(a[indexOfLowestPointX(a)])){
    				if(checkPoint(pv, a[p1], b[q2]) > checkPoint(pv, a[p1], b[q1])){
    					q1 = q2;
    	    			q2 = q1+1;
    	    			if(q2 >= b.length) q2 = 0;
    				}
    				else{
	    				line = new Line(a[p1], b[q1], Color.CYAN);
		    			break;
    				}
    			}else{
	    			p1 = p2;
	    			p2 = p1-1;
	    			if(p2 < 0) p2 = a.length-1;
    			}	
    		}
    	}

    	return line;
    }
    

    private Point[] lowestTangent(Point[] a, Point[] b) {
    	Point pv = null;
    	Point[] lowestPoints = new Point[2];
    	lowestPoints[0]= a[0];
    	lowestPoints[1]= b[b.length-1];
    	
    	pv = new Point(lowestPoints[0].x-10,lowestPoints[0].y);
    	
    	for(int i=1;i<a.length-1;i++){
    		System.out.println("hh"+i);
        	if(checkPoint(pv,lowestPoints[0],a[i+1])<checkPoint(pv,lowestPoints[0],lowestPoints[1]))
        		System.out.println("ifhh"+i);
        		pv = lowestPoints[0];
        		lowestPoints[0] = a[i+1];
    	}
    	pv = new Point(lowestPoints[1].x+10,lowestPoints[1].y);
    	
    	for(int i=b.length-1;i>2;i--){
        	if(checkPoint(lowestPoints[0],b[i-1],pv)>checkPoint(lowestPoints[0],lowestPoints[1],pv))
        		pv = lowestPoints[1];
        		lowestPoints[1] = b[i-1];
    	}
    	

    	
    	
//    	Point[] lowestPoints = new Point[2];
//    	lowestPoints[0]= a[indexOfLowestPointY(a)];
//    	lowestPoints[1]= b[indexOfLowestPointY(b)];
//    	
    	return lowestPoints;
    }
    
    private Point[] highestTangent(Point[] a, Point[] b) {
    	Point[] highestTangent = new Point[2];
    	highestTangent[0]= a[indexOfHighestPointY(a)];
    	highestTangent[1]= b[indexOfHighestPointY(b)];
    	
    	return highestTangent;
    }

    
    private Point[] DevideAndConquer(int lo, int hi, Point[] pt) {
    	Point[] des = null;
    	if((hi-lo) < 2){
    		if(hi-lo > 0){
	    		des = new Point[hi-lo];
	    		for(int i=0;i<(hi-lo);i++)
	    			des[i] = pt[lo+i];
    		}
    	}
    	else{
    		int m=lo + (hi-lo)/2;
	    	des = pt;
	    	
	    	Point[] des1 = DevideAndConquer(lo, m, pt);
	    	Point[] des2 = DevideAndConquer(m, hi, pt);
	    	
	    	if(des1!=null && des2!=null){
		    	
	    		int r = (int)Math.round((Math.random()*255));
		    	int g = (int)Math.round((Math.random()*255));
		    	int b = (int)Math.round((Math.random()*255));

		    	
	    		Point[] lowestPoints = lowestTangent(des1,des2);
		    	
		    	Line line = new Line(lowestPoints[0],lowestPoints[1],new Color(r,g,b));
		    	mainPanel.addPaintable(line);
		    	mainPanel.repaint();
	    	}
	    	
//	    	int loDes1 = indexOfLowestPointY(des1);
//	    	int hiDes1 = indexOfHighestPointY(des1);
//	    	
//	    	Line[] line = new Line[des1.length];
//	    	for(int i=1; i<des1.length; i++)
//	    		line[i-1] = new Line(des1[i-1],des1[i], Color.GREEN);
//	    	line[des1.length-1] = new Line(des1[0],des1[des1.length-1], Color.GREEN);	
//	    	mainPanel.addPaintable(line);	    	
//
//	    	int loDes2 = indexOfLowestPointY(des2);
//	    	int hiDes2 = indexOfHighestPointY(des2);
//	    	
//	 
//	    	line = new Line[des2.length];
//	    	for(int i=1; i<des2.length; i++)
//	    		line[i-1] = new Line(des2[i-1],des2[i], Color.MAGENTA);
//	    	line[des2.length-1] = new Line(des2[0],des2[des2.length-1], Color.MAGENTA);
//	     	mainPanel.addPaintable(line);

	     	
	     	
//	     	mainPanel.addPaintable(new Line(des1[loDes1], des2[loDes2], Color.CYAN));
//	     	mainPanel.addPaintable(new Line(des1[hiDes1], des2[hiDes2], Color.CYAN));
//	    	mainPanel.repaint();
	    	
//	    	Line l2;// = new Line(des1[loDes1], des2[loDes2], Color.CYAN);
//	    	mainPanel.addPaintable(l2);
//	    	l2 = new Line(des1[hiDes1], des2[hiDes2], Color.CYAN);
//	    	mainPanel.addPaintable(l2);
//	    	
//	     	l2 = LowerTangent(des1, des2);
//	     	if(l2 != null)
//	     		mainPanel.addPaintable(l2);
//	    	mainPanel.repaint();
	    	
    	}
    	
//    	if(des!=null){
//    		
//    	
//	    	Line line[] = new Line[des.length];
//	    	int r = (int)Math.round((Math.random()*255));
//	    	int g = (int)Math.round((Math.random()*255));
//	    	int b = (int)Math.round((Math.random()*255));
//	    	
//	    	
//	    	for(int i=1; i<hi-lo-1; i++) {
//	    		line[lo+i-1] = new Line(des[lo+i-1],des[lo+i], new Color(r,g,b));
//	    		mainPanel.addPaintable(line[lo+i-1]);
//	    	}
//	    	mainPanel.repaint();
//	
//	    	try {
//				Thread.sleep(time);
//			} catch (InterruptedException e) {
//				e.printStackTrace();
//			}
//	    	for(int i=1; i<hi-lo-1; i++) {
//	    		mainPanel.removePaintable(line[lo+i-1]);
//	    	}
//	    	mainPanel.repaint();
//	
//    	}

		return des;
    }
/** computes hull points with the Devide and Conquer algorithm
 */
private void DevideAndConquer(){
	Point[] pt = DevideAndConquer(0, p.length, p);
	
//	Line[] line = new Line[pt.length];
//	
//	for(int i=1; i<pt.length; i++)
//		line[i-1] = new Line(pt[i-1],pt[i], Color.RED);
//		
//	
//	line[pt.length-1] = new Line(pt[0],pt[pt.length-1], Color.RED);
//		
//	mainPanel.addPaintable(line);
//	mainPanel.repaint();
}   
    
public double checkPoint(Point a, Point b, Point c){
    	
    	double ax = a.x - b.x;
    	double ay = a.y - b.y;
    	double bx = c.x - b.x;
    	double by = c.y - b.y;
    	
    	return Math.acos((ax*bx + ay*by) / (Math.sqrt(ax*ax + ay*ay) * Math.sqrt(bx*bx + by*by)));	
    }

	public void run() {
		int i=indexOfLowestPointX(p);   	
		exchange(0, i);
		sort();
		DevideAndConquer();	
//		this.notifyAll();
	}

	@Override
	public void set(int m) {
		// TODO Auto-generated method stub
		
	}
}
