package ala2D;

import java.awt.Color;
import java.util.LinkedList;

public class GrahamScan extends ConvexHullAlgorithem
{
    private Point[] p; // array of points
    //private static int n;     // number of points
    //private static int m;     // number of points of the polygon
	private LinkedList<Point> l = new LinkedList<Point>();
	private LinkedList<Line> l2 = new LinkedList<Line>();
	private MainPanel mainPanel = MainPanel.getInstance();
	private int time = 100;
	
	
    public void set(Point[] q) {
//        p=new Point[q.length];
//        for(int i=0;i<q.length;i++)
//        	p[i]=q[i];
    	p=q;
    }
    
    public Point[] get(){

    	Point[] result = new Point[l.size()];
    	for(int i=0;i<l.size();i++)
    		result[i] = l.get(i);
    	return result;
    }

    public void setTime(int time){
    	this.time = time;
    }

    /** exchanges two points p[i] and p[j]
     */
    private void exchange(int i, int j)
    {
        Point t=p[i];
        p[i]=p[j];
        p[j]=t;
    }


    /** computes coordinates of all points relative to point p0
     */
    private void makeRelativeTo(Point p0)
    {
        int i;
        Point p1=new Point(p0); // notwendig, weil p0 in p[] sein kann
        for (i=0; i<p.length; i++)
            p[i].makeRelativeTo(p1);
    }


    /** determines index of a point with minimum y-coordinate
     */
    private int indexOfLowestPoint()
    {
        int i, min=0;
        for (i=1; i<p.length; i++)
            if (p[i].isLower(p[min]))
                min=i;
        return min;
    }


    /** tests if the i-th node of the polygon is convex
     */
    private boolean isConvex(int i)
    {
        return p[i].relativeTo(p[i-1]).isLess(p[i+1].relativeTo(p[i-1]));
    }


    /** sorts points except p[0] according to their angle
     */
    private void sort()
    {
    	quicksort(1, p.length - 1); // ohne Punkt 0
    }


    /** sorts points according to their angle
     */
    protected void quicksort(int lo, int hi)
    {
        int i=lo, j=hi;
        Point q=p[(lo+hi)/2];
        while (i<=j)
        {
            while (p[i].isLess(q)) i++;
            while (q.isLess(p[j])) j--;
            if (i<=j) exchange(i++, j--);
        }
        if (lo<j) quicksort(lo, j);
        if (i<hi) quicksort(i, hi);
    }

/** computes hull points with the Graham-Scan algorithm
 */
public void grahamScan()
{
	int i=indexOfLowestPoint();
	//l.add(p[i]);    	
	exchange(0, i);
	Point q=new Point(p[0]);
	makeRelativeTo(q);
	sort();
	makeRelativeTo(q.reversed());
	Point pv = new Point(p[0].x-20, p[0].y, Color.CYAN);
	
   	mainPanel.addPaintable(pv);
   	
   	Line[] line = new Line[p.length];
	
	for(i=1; i<p.length; i++)
		line[i-1] = new Line(p[i-1],p[i], Color.BLACK);
		
	
	line[p.length-1] = new Line(p[p.length-1],p[0], Color.BLACK);
		
	mainPanel.addPaintable(line);
	mainPanel.repaint();
   	
    int k=3,m=3;
    
    Line searchLine=null, posibleLine = null;
    posibleLine = new Line(p[0], p[1], Color.ORANGE);
    mainPanel.addPaintable(posibleLine);
    l2.add(posibleLine);
    //l.add(p[1]);
    posibleLine = new Line(p[1], p[2], Color.ORANGE);
    mainPanel.addPaintable(posibleLine);
    l2.add(posibleLine);
    //l.add(p[2]);
   
    if (p.length>3)
    	searchLine = new Line(p[2], p[3], Color.MAGENTA);
 
    	
    
//    mainPanel.addPaintable(searchLine);
    l2.add(posibleLine);
    //l.add(p[3]);
//    mainPanel.revalidate();
//	mainPanel.repaint();
    
    while (k<p.length)
    {
    	//l.removeLast();
    	//l.add(p[k]);
    	exchange(m, k);
    	
        while (!isConvex(m-1)){

        	mainPanel.removePaintable(l2.getLast());
        	l2.removeLast();
        	//l.removeLast();
        	if(searchLine!=null)
        		mainPanel.removePaintable(searchLine);
        	searchLine = new Line(p[m-2], p[m], Color.MAGENTA); 
        	mainPanel.addPaintable(searchLine);
        	mainPanel.revalidate();
        	mainPanel.repaint();
        	
        	exchange(m-1, m--);
        }
        posibleLine = new Line(p[m-1], p[m], Color.ORANGE);
    	mainPanel.addPaintable(posibleLine);
    	l2.add(posibleLine);
    	//l.add(p[m]);
    	mainPanel.revalidate();
    	mainPanel.repaint();
    	
        m++;
        k++;
        
        try {
			Thread.sleep(time);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    mainPanel.addPaintable(new Line(p[m-1], p[0], Color.ORANGE));
	if(searchLine!=null)
		mainPanel.removePaintable(searchLine);
    //l.removeLast();
    
    for(i=0;i<m;i++)
    	l.add(p[i]);
    
    for(i=0;i<line.length;i++)
    	mainPanel.removePaintable(line[i]);
    
//    mainPanel.removePaintable(line[0]);
//    mainPanel.removePaintable(line[line.length-1]);
    mainPanel.revalidate();
	mainPanel.repaint();
}   
    
public double checkPoint(Point a, Point b, Point c){
    	
    	double ax = a.x - b.x;
    	double ay = a.y - b.y;
    	double bx = c.x - b.x;
    	double by = c.y - b.y;
    	
    	return Math.acos((ax*bx + ay*by) / (Math.sqrt(ax*ax + ay*ay) * Math.sqrt(bx*bx + by*by)));	
    }

	public void run() {
		grahamScan();
//		this.notifyAll();
	}

	@Override
	public void set(int m) {
		// TODO Auto-generated method stub
		
	}
	
}   // end class GrahamScan
