package ala2D;
import java.awt.Color;
import java.awt.Graphics;

class Point implements Paintable
{
    public double x, y;    // Koordinaten
    public int nr;
    public boolean marked=false;
    private Color col = Color.BLUE;
    public static int gr = 6;
    
    
    
	public void paintMe(Graphics g) {
		g.setColor(col);
		//g.drawOval((int)x,(int)y,5,5);
		g.fillOval((int)x-gr/2,(int)y-gr/2,gr,gr);
	}

    public Point(int nr, double x, double y)
    {
        this.nr=nr;
        this.x=x;
        this.y=y;
    }


    public Point(double x, double y)
    {
        this(0, x, y);
    }


    public Point(Point p)
    {
        this(0, p.x, p.y);
    }
    
    public Point(double x, double y, Color col)
    {
        this(0, x, y);
        this.col = col;
    }


    public Point()
    {
        this(0, 0, 0);
    }


    /** berechnet das Quadrat des Euklidischen Abstands
     *  dieses Punkts zum Nullpunkt
     */
    public double distance2()
    {
        return x*x + y*y;
    }


    /** transformiert diesen Punkt
     *  relativ zu p als Nullpunkt
     */
    public void makeRelativeTo(Point p)
    {
        x-=p.x;
        y-=p.y;
    }


    /** erzeugt einen neuen Punkt, der diesen Punkt
     *  relativ zu p darstellt
     */
    public Point relativeTo(Point p)
    {
        return new Point(x-p.x, y-p.y);
    }


    /** erzeut einen am Nullpunkt zu diesem Punkt gespiegelten Punkt
     */
    public Point reversed()
    {
        return new Point(-x, -y);
    }


    /** gibt true zurck, wenn der Ortsvektor dieses Punkts
     *  einen kleineren Winkel als der Ortsvektor von p hat
     *  oder, bei gleichem Winkel, wenn er krzer ist als p
     */
    public boolean isLess(Point p)
    {
        double f=x*p.y - p.x*y;    // cross product
        return f>0 || f==0 && distance2()<p.distance2();
    }


    /** gibt true zurck, wenn dieser Punkt eine kleinere
     *  y-Koordinate als p hat oder, bei gleicher y-Koordinate,
     *  wenn er eine kleinere x-Koordinate als p hat
     */
    public boolean isLower(Point p)
    {
        return y<p.y || y==p.y && x<p.x;
    }
    
    public boolean isLowerX(Point p)
    {
        return x<p.x || x==p.x && y<p.y;
    }

	public String getKey() {
		return "P("+x+";"+y+")";
	}
    



}    // end class Point
